#ifndef CORE_ARRAY_HPP
#define CORE_ARRAY_HPP

#include "core/utils/ArrayString.hpp"

namespace Core {
    template<typename T, int N>
    class Array final {
        static_assert(N > 0, "Array size must be positive");
        T data[static_cast<unsigned int>(N)];

    public:
        constexpr Array() = default;

        constexpr Array(const T& t) {
            fill(t);
        }

        constexpr void fill(const T& t) {
            for(int i = 0; i < N; i++) {
                data[i] = t;
            }
        }

        constexpr T& operator[](int index) {
            return data[index];
        }

        constexpr const T& operator[](int index) const {
            return data[index];
        }

        constexpr T* begin() {
            return data;
        }

        constexpr T* end() {
            return data + N;
        }

        constexpr const T* begin() const {
            return data;
        }

        constexpr const T* end() const {
            return data + N;
        }

        static consteval int getLength() {
            return N;
        }

        template<typename String>
        check_return Error toString(String& s) const {
            return Core::toString(s, *this);
        }
    };
}

#endif